﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Olap.FlatData;
using Infragistics.Olap;
using Infragistics.Controls.Grids;
using Infragistics.Olap.Data;
using Infragistics.Windows;
using System.Windows.Controls.Primitives;
using System.Diagnostics;
using Infragistics.Olap.Data.Base;
using System.Linq.Expressions;
using System.Threading;

namespace TestApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public static int LevelDepth;
        private ManualResetEvent manualResetEvent;
        FlatDataSource dataSource;
        public MainWindow()
        {
            InitializeComponent();

            Data = new ObservableCollection<Data>();
            Random r = new Random();

            for (int i = 0; i < 50; i++)
            {
                

                int days = i % 5;
                Data.Add(new Data
                {
                    Test = "Test" + r.Next(10),
                    Result = r.Next(5000),
                    Date = DateTime.Now.AddDays(days + 10)
                });
            }

        }

        public ObservableCollection<Data> Data { get; set; }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            string mStr = "Result";

            dataSource = new FlatDataSource()
            {
                ItemsSource = Data,
                Cube = DataSourceBase.GenerateInitialCube("Test Results"),
                Rows = DataSourceBase.GenerateInitialItems("[Test].[Test]"),
                Columns = DataSourceBase.GenerateInitialItems("[Date].[Date]"),
                Measures = DataSourceBase.GenerateInitialItems(mStr)
               
            };
            
            xamPivotGrid1.DataSource = dataSource;
        }

        private void Button_Click(object sender, RoutedEventArgs args)
        {
	    //Here you should set the Depth of your Data
            MainWindow.LevelDepth = 5;

            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += worker_DoWork;
            worker.RunWorkerCompleted += (s, e) =>
            {
                this.xamPivotGrid1.DataSource.ResultChanged += DataSource_ResultChanged;
                this.xamPivotGrid1.DataSource.DeferredLayoutUpdate = false;
            };
            this.xamPivotGrid1.DataSource.DeferredLayoutUpdate = true;
            worker.RunWorkerAsync(this.xamPivotGrid1.DataSource);
            
        }


        void DataSource_ResultChanged(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            DataSourceBase dataSourceBase = (DataSourceBase)sender;
            dataSourceBase.ResultChanged -= this.DataSource_ResultChanged;

            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.DrillDown;
            dataSourceBase.UpdateVisualMembers(Infragistics.Olap.ExecutionContext.Rows);
            dataSourceBase.UpdateVisualMembers(Infragistics.Olap.ExecutionContext.Columns);
            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.SingleLevelOnly;

            dataSourceBase.RefreshGrid();
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            DataSourceBase dataSource = e.Argument as DataSourceBase;

            IFilterViewModel fvm = FindDateViewModel(dataSource);
            if (fvm != null)
            {
                fvm.LoadFilterMembersCompleted += (sender1, e1) =>
                {
                    manualResetEvent.Set();
                };

                if (fvm != null && fvm.FilterMembers != null)
                {
                    foreach (IFilterMember member in fvm.FilterMembers)
                    {
                        DrillIn(fvm, member);
                    }
                }
            }
            else
            {
                this.Dispatcher.Invoke(
                    System.Windows.Threading.DispatcherPriority.Normal,
                    new Action(
                        delegate()
                        {
                            MessageBox.Show("The Date hierarchy is not added to the row or column axises");
                        }));
            }
        }

        private IFilterViewModel FindDateViewModel(DataSourceBase dataSource)
        {

            for (int i = 0; i < dataSource.Rows.Count; i++)
            {
                if ((dataSource.Rows[i] as IFilterViewModel).Caption == "Date")
                {
                    return dataSource.Rows[i] as IFilterViewModel;
                }
            }

            for (int i = 0; i < dataSource.Columns.Count; i++)
            {
                if ((dataSource.Columns[i] as IFilterViewModel).Caption == "Date")
                {
                    return dataSource.Columns[i] as IFilterViewModel;
                }
            }

            return null;
        }

        private void DrillIn(IFilterViewModel filterViewModel, IFilterMember filterMember)
        {
            manualResetEvent = new ManualResetEvent(false);

            //Condition to drill in further
            if (filterMember.Member.LevelDepth < MainWindow.LevelDepth)
            {
                this.Dispatcher.Invoke(
                    System.Windows.Threading.DispatcherPriority.Normal,
                    new Action(
                        delegate()
                        {
                            filterMember.IsExpanded = true;
                        }));

                if (filterMember.FilterMembers == null)
                    manualResetEvent.WaitOne();

                foreach (IFilterMember member in filterMember.FilterMembers)
                {
                    DrillIn(filterViewModel, member);
                }
            }
            else
            {
                this.Dispatcher.Invoke(
                    System.Windows.Threading.DispatcherPriority.Normal,
                    new Action(
                        delegate()
                        {
                            filterMember.IsExpanded = false;
                        }));

            }
        }
        
    }

   


    public class Data : INotifyPropertyChanged
    {

        public string Test
        {
            get { return test; }
            set
            {
                test = value;
                OnPropertyChanged("Test");
            }
        }

        public int Result
        {
            get { return result; }
            set
            {
                result = value;
                OnPropertyChanged("Result");
            }
        }

        public DateTime Date
        {
            get { return date; }
            set
            {
                date = value;
                OnPropertyChanged("Date");
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;
        private string test;
        private int result;
        private DateTime date;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }
    }

}
